@@ -1,9 +1,13 @@
-use std::{io::Read, sync::Arc};
+use std::{
+    io::{Read, Write},
+    sync::Arc,
+};
 
 use anyhow::Context;
 use config::Config;
 use env_logger::Env;
 
+use flate2::{write::GzEncoder, Compression};
 use jemallocator::Jemalloc;
 
 use structopt::StructOpt;
@@ -51,6 +55,25 @@ fn main() -> anyhow::Result<()> {
         "read configuration file:\n{}",
         serde_json::to_string_pretty(&config)?
     );
+
+    if let Some(trace) = config.sosistab_trace().as_ref() {
+        let (send, recv) = flume::unbounded();
+        log::info!("writing gzipped sosistab traces to {:?}", trace);
+        sosistab::init_packet_tracing(move |line| send.send(line).unwrap());
+        let trace = trace.clone();
+        std::thread::Builder::new()
+            .name("trace-writer".into())
+            .spawn(move || {
+                let file = std::fs::File::create(&trace).expect("cannot create trace file");
+                let mut file = GzEncoder::new(file, Compression::best());
+                loop {
+                    let line = recv.recv().unwrap();
+                    writeln!(file, "{}", line).expect("cannot write line to file");
+                }
+            })
+            .unwrap();
+    }
+
     if let Some(nat_interface) = config.nat_external_iface().as_ref() {
         config_iptables(nat_interface)?;
     }
@@ -70,7 +93,7 @@ iptables -t nat -F
 
 iptables -t nat -A PREROUTING -i tun-geph -p tcp --syn -j REDIRECT --match multiport --dports 80,443,8080 --to-ports 10000
 
-iptables -t nat -A POSTROUTING -o $INTERFACE -j MASQUERADE --random
+iptables -t nat -A POSTROUTING -o $INTERFACE -j MASQUERADE --random-fully
 iptables -A FORWARD -i $INTERFACE -o tun-geph -m state --state RELATED,ESTABLISHED -j ACCEPT
 iptables -A FORWARD -i tun-geph -o $INTERFACE -j ACCEPT
 iptables -A FORWARD -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --set-mss 1240
